package nl.sbdeveloper.mctpaudio.listener;

import com.mpatric.mp3agic.InvalidDataException;
import com.mpatric.mp3agic.UnsupportedTagException;
import com.sk89q.worldguard.protection.regions.ProtectedRegion;
import net.raidstone.wgevents.events.RegionEnteredEvent;
import net.raidstone.wgevents.events.RegionLeftEvent;
import nl.sbdeveloper.mctpaudio.MCTPAudio;
import nl.sbdeveloper.mctpaudio.api.AudioType;
import nl.sbdeveloper.mctpaudio.managers.PinManager;
import nl.sbdeveloper.mctpaudio.managers.WGManager;
import nl.sbdeveloper.mctpaudio.utils.HeadUtil;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.EventHandler;
import org.bukkit.event.Listener;
import org.bukkit.event.player.PlayerJoinEvent;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.scheduler.BukkitRunnable;
import org.json.simple.JSONObject;

import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.UUID;

/* Copyright (C) McThemeParks - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written by Stijn Bannink <stijnbannink23@gmail.com>, July 2020
 */

/**
 * This class handles playing numbers in a region.
 * This will NOT be used for region much, only if you start playing music in a region with the @a[region=...] selector.
 */
public class PlayInRegionHandler implements Listener {
    private final String region;
    private final String url;

    private final List<UUID> players;
    private int currentTick = 0;
    private boolean isListening = true;

    public PlayInRegionHandler(String region, String url, List<UUID> players) {
        this.region = region;
        this.url = url;
        this.players = players;

        load();
        Bukkit.getPluginManager().registerEvents(this, MCTPAudio.getPlugin());
    }

    /**
     * Sends the song to every audio client, and starts it.
     */
    private void load() {
        int ticks;
        try {
            ticks = HeadUtil.getTicksOfFile(url);
        } catch (IOException | InvalidDataException | UnsupportedTagException e) {
            e.printStackTrace();
            return;
        }

        new BukkitRunnable() {
            @Override
            public void run() {
                currentTick++;
                if (currentTick == ticks) {
                    isListening = false;
                    cancel();
                }
            }
        }.runTaskTimer(MCTPAudio.getPlugin(), 0L, 1L);

        Player p;
        JSONObject data;
        Iterator<UUID> uuidS = players.iterator();
        while (uuidS.hasNext()) {
            p = Bukkit.getPlayer(uuidS.next());
            if (p == null || !PinManager.hasPin(p.getUniqueId())) {
                uuidS.remove();
                continue;
            }

            data = new JSONObject();
            data.put("task", AudioType.SFX.name());
            data.put("path", url);
            data.put("uuid", p.getUniqueId().toString().replace("-", ""));
            data.put("play", false);
            MCTPAudio.getClient().sendData(data);
        }

        startForAll();
    }

    /**
     * Starts the song for every player.
     */
    private void startForAll() {
        players.forEach(this::start);
    }

    private void start(UUID pUUID) {
        start(pUUID, 0);
    }

    /**
     * Starts the song for a Player.
     * @param pUUID The player to start it for.
     */
    private void start(UUID pUUID, int ms) {
        if (!isListening) return;

        Player p = Bukkit.getPlayer(pUUID);
        if (p == null) return;
        if (!PinManager.hasPin(p.getUniqueId())) return;

        JSONObject data = new JSONObject();
        data.put("task", AudioType.SFX.name());
        data.put("path", url);
        data.put("uuid", p.getUniqueId().toString().replace("-", ""));
        data.put("play", true);
        data.put("at", ms);
        MCTPAudio.getClient().sendData(data);
    }

    /**
     * Stops the song for a Player.
     * @param pUUID The player to stop it for.
     */
    private void stop(UUID pUUID) {
        if (!isListening) return;

        Player p = Bukkit.getPlayer(pUUID);
        if (p == null) return;
        if (!PinManager.hasPin(p.getUniqueId())) return;

        JSONObject data = new JSONObject();
        data.put("task", AudioType.SFX.name());
        data.put("path", url);
        data.put("uuid", p.getUniqueId().toString().replace("-", ""));
        data.put("play", false);
        MCTPAudio.getClient().sendData(data);
    }

    @EventHandler
    private void onEnterRegion(RegionEnteredEvent e) {
        if (e.getPlayer() == null) return;

        if (e.getRegionName().equals(region)) {
            if (!players.contains(e.getPlayer().getUniqueId())) {
                players.add(e.getPlayer().getUniqueId());
                start(e.getPlayer().getUniqueId(), getMs());
            } else {
                start(e.getPlayer().getUniqueId());
            }
        }
    }

    @EventHandler
    private void onLeaveRegion(RegionLeftEvent e) {
        if (e.getPlayer() == null) return;

        if (e.getRegionName().equals(region)) {
            if (!players.contains(e.getPlayer().getUniqueId())) return;
            stop(e.getPlayer().getUniqueId());
        }
    }

    @EventHandler
    private void onJoin(PlayerJoinEvent e) {
        List<ProtectedRegion> regionsIn = WGManager.getRegionsIn(e.getPlayer().getLocation());
        if (regionsIn.stream().anyMatch(reg -> reg.getId().equals(region))) {
            players.add(e.getPlayer().getUniqueId());
            start(e.getPlayer().getUniqueId(), getMs());
        }
    }

    @EventHandler
    private void onQuit(PlayerQuitEvent e) {
        if (!players.contains(e.getPlayer().getUniqueId())) return;
        stop(e.getPlayer().getUniqueId());
        players.remove(e.getPlayer().getUniqueId());
    }

    private int getMs() {
        return Math.round(currentTick * 50);
    }
}
